#!/usr/bin/python

import sys
import json

def write_default_from_schema( path_to_schema ):
    """
    This very simple funtion takes a DTk schema json file and creates a default config.json file. 
    It's as good as the schema it's given. Note that this is designed to work with a schema from
    a disease-specific build, otherwise it may contain a lot of params from other disease types.
    """
    default_config = { "parameters": {} }
    schema = json.loads( open( path_to_schema ).read() )
    for group in schema["config"]:
        for param in schema["config"][group]:
            if 'default' in schema["config"][group][param]:
                value = schema["config"][group][param]['default']
                default_config[ "parameters" ][ param ] = value

    with open( "default_config.json", "w" ) as outfile:
        json.dump( default_config, outfile, sort_keys=True, indent=4 )
    print( "Wrote 'default_config.json'file." )

if __name__=="__main__":
    if len( sys.argv ) == 1:
        print( "Usage: emod_api.default_from_schema <schema.json>" )
    else:
        write_default_from_schema( sys.argv[1] )

